︠8dd342dc-113f-47fb-ab41-04c89e930989si︠
#Evaluate this cell for Exercise 1
%hide
%auto
def dde_solve(dde, statevar, delayedvars, history, tmax, timestep):
    # Check validity of delays.
    if min(delayedvars.values()) < 0:
        raise ValueError("This function will not work with negative delays. "
            "Consider consulting a fortune teller instead.")
    if any(val<timestep for val in delayedvars.values()):
        raise ValueError("Time step should not be larger than delay.")

    # Set up variables and delays.
    delayedvars = delayedvars.items()
    dde = dde.subs({v: statevar for v, delay in delayedvars if delay == 0})
    delayedvars = [(v, delay) for v, delay in delayedvars if delay != 0]
    allvars = [str(statevar)] + [str(v) for v, delay in delayedvars]
    delays = [delay for v, delay in delayedvars]

    # Set up fast functions.
    dde_func = fast_float(dde, *allvars)
    history_func = fast_float(history, "t")

    # Adjust the timestep if necessary
    mindelay = min(delays) if delays else timestep
    timestepcorrectionfactor = ceil(timestep / mindelay)
    timestep /= timestepcorrectionfactor

    # A function to perform history lookups.
    def lookup(t):
        """Does a history lookup at each delay from t, stores result in allvars[1:]"""
        for i, delay in enumerate(delays):
            if t - delay <= 0:
                allvars[i+1] = history_func(t - delay)
            else:
                r = (t - delay) / timestep
                n = floor(r)
                r -= n
                allvars[i+1] = result[n]*(1 - r) + result[n + 1]*r

    # Set up for the first iteration.
    result = [history_func(0)]
    lookup(0)
    for t in sxrange(0, tmax - timestep, timestep):
        # Compute k1. Note history lookup has already been done.
        allvars[0] = result[-1]
        k1 = dde_func(*allvars)
        # Compute k2.
        lookup(t + timestep/2)
        allvars[0] += timestep/2 * k1
        k2 = dde_func(*allvars)
        # Compute k3. Note history lookup has already been done.
        allvars[0] = result[-1] + timestep/2 * k2
        k3 = dde_func(*allvars)
        # Compute k4.
        lookup(t + timestep)
        allvars[0] = result[-1] + timestep * k3
        k4 = dde_func(*allvars)
        # Finally, compute the RK4 weighted average.
        result.append(result[-1] + (k1 + 2*k2 + 2*k3 + k4)/6 * timestep)
    return result[::timestepcorrectionfactor]
︡1ea7cfed-6cfa-4d35-a151-2031c94e9071︡
︠ccbe321e-24bf-4c51-a616-cdab80f9604eis︠
%hide 
%md
## <span style='color:brown'> Dictionary </span>

When we want to save multiple variables, we use "List"

### [variable_1, variable_2,........,variable_k]
#### <span style='color:red'> Problem: We need to know the "Index" of the data</span>
#### <span style='color:blue'> Can we storage the data with some description? </span>
︡2bc065d2-65f5-40c7-a71c-effc2a1a4a77︡{"hide":"input"}︡{"md":"## <span style='color:brown'> Dictionary </span>\n\nWhen we want to save multiple variables, we use \"List\"\n\n### [variable_1, variable_2,........,variable_k]\n#### <span style='color:red'> Problem: We need to know the \"Index\" of the data</span>\n#### <span style='color:blue'> Can we storage the data with some description? </span>"}︡{"done":true}
︠55ff505e-8803-4a18-9711-da54ca21b416︠
# this is an example for midterm score
midDic = {'John':100,'Jack':70,'Peter':88,'Jane':66}
midDic['John']
︡aaae371a-ab7d-40ce-9c32-eef009f910e7︡{"stdout":"100\n"}︡{"done":true}
︠b6c02fad-bf0b-4fcb-a6bb-9f534c8150c9is︠
%hide
%md
## <span style='color:brown'> Delay Differential Equations </span>
Recall Homework Exercise 4.2.4
︡10f474b3-807a-4a75-ba9b-c800a44c4306︡{"hide":"input"}︡{"md":"## <span style='color:brown'> Delay Differential Equations </span>\nRecall Homework Exercise 4.2.4"}︡{"done":true}
︠192fc8b7-1768-4f89-9098-8bb55deaf1afsi︠
%hide
%latex
\begin{flushleft}
$Y'(t)=16 Y(t-2)+8Y(t)$
\end{flushleft}
︡a6b2c783-2b8e-4bc7-bf6a-aaf3922df611︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpuXnxJ4.png","show":true,"text":null,"uuid":"4a8ff861-bbf3-45d1-8dfa-3b37caf5d86d"},"once":false}︡{"done":true}
︠2ca3752c-a039-4a92-8709-de23aea8e6f2is︠
%hide
%md
#### <span style='color:red'> What does this mean? </span>
If I want to know Y'(10), by given you the Y's history.
︡21f0beac-a859-43f2-ac90-66cc58c79b2b︡{"hide":"input"}︡{"md":"#### <span style='color:red'> What does this mean? </span>\nIf I want to know Y'(10), by given you the Y's history."}︡{"done":true}
︠37eb6266-b28e-49ba-8bd3-7458c42c3266si︠
%hide
%latex
\begin{flushleft}
(a) Y'(10) = 16Y(8)+8Y(10)\\
(b) Y'(10) = 16Y(12)+8Y(10)\\
(c) Y'(10) = 16Y(10)+8Y(8)\\
\end{flushleft}
︡0bb1d367-8488-4492-bc66-084ff2099d9a︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpHTdClO.png","show":true,"text":null,"uuid":"454ae2fa-6923-41b7-88e5-496a5547cf51"},"once":false}︡{"done":true}
︠6f61d5aa-489a-47e4-ad18-011224ab5b8bis︠
%hide
%md
## <span style='color:red'> Now let's solve it </span>

Function we are going to use!!
```
sol=dde_solve(equation, statevar,delayedvars, history, tmax, timestep)
```
︡18e50109-2b78-4765-9a7a-fd12986673b3︡{"hide":"input"}︡{"md":"## <span style='color:red'> Now let's solve it </span>\n\nFunction we are going to use!!\n```\nsol=dde_solve(equation, statevar,delayedvars, history, tmax, timestep)\n```"}︡{"done":true}
︠40e16abf-d3f4-4c2e-80dc-7c396d0e11a0is︠
%hide
%latex
\begin{flushleft}
$Y'(t)=2Y^3(t-2)+2Y^2(t-1)-8Y(t)$\\
Y(t) = 1 if t<0\\
Simulate from 0 to 10!!!
\end{flushleft}
︡b230aa8a-3db2-435d-96d8-3f73bab87c41︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpqC5mVO.png","show":true,"text":null,"uuid":"694619b3-7322-41d9-be5c-a7d683bcb7c6"},"once":false}︡{"done":true}
︠5b71610b-0de9-44dc-b789-863b8a01056bs︠
Y = var('Y')
Y_d = var('Y_d')
Y_2d = var('Y_2d')
t = var('t')
delayvar_dic = {'Y_2d':2,'Y_d':1}
time = srange(0,5,0.01)
sol = dde_solve(2*Y_2d^3+2*Y_d^2-8*Y,statevar=Y,delayedvars=delayvar_dic,history=1,tmax=5,timestep=0.01)
fig = list_plot(zip(time,sol),plotjoined=True)
show(fig)
︡6d0e8902-17d7-4a2f-ba0e-5dd5a7cc4088︡{"file":{"filename":"/home/user/.sage/temp/project-abb83c66-a9f1-488b-80bd-fd40b9e4a2fc/1470/tmp_qy6KlM.svg","show":true,"text":null,"uuid":"464e19b5-b521-4a48-9f5c-2461294c9a53"},"once":false}︡{"done":true}
︠967acc42-270d-4ca5-aacc-3d51822355f5is︠
%hide
%md
## <span style = 'color:red'> Steps to run simulation with time-delay </span>
- Generate individual symbolic variables for state and delay
- Use a dictionary to connect delay time with time-delay state variables
- Give a function describes the states before t = 0
- Give total simulation time and time step
︡ecb71f66-9095-4758-9369-9cb1aacdb9ef︡{"hide":"input"}︡{"md":"## <span style = 'color:red'> Steps to run simulation with time-delay </span>\n- Generate individual symbolic variables for state and delay\n- Use a dictionary to connect delay time with time-delay state variables\n- Give a function describes the states before t = 0\n- Give total simulation time and time step"}︡{"done":true}
︠49680ae6-0cf0-4813-8fca-58e0f709f7fdis︠
%hide
%md
## <span style='color:brown'> Oscillation </span>
### <span style='color:blue'> Sigmoid Function </span>
︡0d7ac989-cd11-4ef5-a49e-2cfb2a534c8f︡{"hide":"input"}︡{"md":"## <span style='color:brown'> Oscillation </span>\n### <span style='color:blue'> Sigmoid Function </span>"}︡{"done":true}
︠10fa577b-295c-4fbe-bf28-9ff922da5febis︠
%hide
%latex
\begin{flushleft}
$\frac{1}{1+G^n}$ and $\frac{G^n}{1+G^n}$
\end{flushleft}
︡1b8cde07-02c3-40e3-a885-392a9852eeec︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpXJr6Qs.png","show":true,"text":null,"uuid":"82a78584-70fb-4fd8-8221-608ab27cebdb"},"once":false}︡{"done":true}
︠f8b2cc53-efa1-44c0-a638-4e2cef432fc5s︠
G = var('G')
fig1 = plot(1/(1+G^10),(G,0.001,5),legend_label='Negative Sigmoid')
fig2 = plot(G^10/(1+G^10),(G,0.001,5),color='red',legend_label='Positive Sigmoid')
show(fig1+fig2)
︡ba4ac5aa-b602-4baa-8dc4-6c5bb27ad400︡{"file":{"filename":"/home/user/.sage/temp/project-abb83c66-a9f1-488b-80bd-fd40b9e4a2fc/643/tmp_Sygpw2.svg","show":true,"text":null,"uuid":"d7cb3610-9751-4128-b5e7-935e3426e410"},"once":false}︡{"done":true}
︠6beaf1ab-2722-4382-8de8-83e54c487f2dsi︠
%hide
%md
## How to cause oscillation?
- Too small-> Increase, Too big -> Decrease
- Will not reach stable equilibrium points on the limit cycle
︡3a3caf0a-329e-418b-ab1b-2797cd040b1a︡{"hide":"input"}︡{"md":"## How to cause oscillation?\n- Too small-> Increase, Too big -> Decrease\n- Will not reach stable equilibrium points on the limit cycle"}︡{"done":true}
︠1d68ac9a-7654-43c8-a7f0-c145c0dbdf86is︠
%hide
%latex
\begin{flushleft}
$X'=V$\\
$V'=-X-(aV^3-V)$

\end{flushleft}
︡541ba1a7-4b77-4344-89b5-d6d4dc469a2c︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmpHv9zyr.png","show":true,"text":null,"uuid":"41fde8ab-3d82-468b-89a7-435c5c3efdae"},"once":false}︡{"done":true}
︠c71eb31b-7453-4e18-b081-76b1acf7e97dis︠
%hide
%md
## What will happen if Mackey-Glass model has no delay? 

︡e878e575-009c-4974-85d6-dbb6d5d5f8b4︡{"hide":"input"}︡{"md":"## What will happen if Mackey-Glass model has no delay? "}︡{"done":true}
︠31b2e4dc-ed94-443f-8273-282fafd5d2acis︠
%hide
%latex
\begin{flushleft}
$x'(t)=L-\frac{V_{max}x(t-\tau)^n}{h-x(t-\tau)^n}ax(t)$ =>$x'(t)=L-\frac{V_{max}x(t)^n}{h-x(t)^n}ax(t)$
\end{flushleft}

︡39647698-56ae-4264-ba8e-052f1158b927︡{"hide":"input"}︡{"file":{"filename":"/tmp/tmp8UIRiS.png","show":true,"text":null,"uuid":"4cd89a26-293d-47c2-9f81-74719e357e73"},"once":false}︡{"done":true}
︠011a29a1-e586-47ef-b498-8c14ce501ccfs︠
Vmax = 80
L = 6
a = 0.2
n = 8
h = 400
x = var('x')
t = srange(0,2,0.01)
sol = desolve_odeint(L-Vmax*x^n/(h+x^n)*a*x,ics = 0.5,dvars = x,times=t)
fig1 = list_plot(zip(t,sol),plotjoined = True,color = 'red')
sol = desolve_odeint(L-Vmax*x^n/(h+x^n)*a*x,ics = 5,dvars = x,times=t)
fig2 = list_plot(zip(t,sol),plotjoined = True,color = 'blue')
fig1+fig2
︡7f77dfd0-8944-46bb-b0af-e4f00765c1b4︡{"file":{"filename":"/home/user/.sage/temp/project-abb83c66-a9f1-488b-80bd-fd40b9e4a2fc/643/tmp_dUDnPv.svg","show":true,"text":null,"uuid":"5b5479e5-08b7-4d77-8d14-c1a4289f1ca2"},"once":false}︡{"done":true}
︠247db7aa-5051-4180-8805-7c8ed500cc5d︠










